/*****************************************************************************/

/*
 *      rxtx.c  --  routines for file selection,receiving, storing, sending
 *	and text macro management
 * 	for the simple HF terminal program.
 *
 *      Copyright (C) 1996  Thomas Sailer (sailer@ife.ee.ethz.ch)
 *        Swiss Federal Institute of Technology (ETH), Electronics Lab
 *	modified 2000-2004 by Axel Krause & Gnther Montag (dl4mge@darc.de).
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

/*****************************************************************************/

#include "hft.h"
#include "rxtx.h"

/* --------------------------------------------------------------------- */
int beacon_on = 0;
char specialfile[256], rxfilehead[256], rxfilefoot[256];
char keytext[MAXKEY];
const char *fixmagic = "\nYour MAGIC KEYS:\n[MYCALL] [OP] [QTH] [LOC] [RIG] [PWR] [ANT] [WWW]\nwill be replaced by your entries from 'Personal Edit'.\n[TIME] will print a timestamp.\n[ACALL] (Amtor), [PCALL] = [CALL] (Pactor, also for RTTY), [GCALL] (Gtor)\nis your QSO-Partner's call, taken from 'Parameters'.\n[LTIME] [LCALL] [LNAME] [LQTH] [LRSTIN] [LRSTOUT] [LNOTES] [LMODE] [LBAND]\nwill be replaced by last log entry.\nYou can use the well-known 'Strg-X, Strg-C, Strg-V to cut, copy and insert from one window to another.\nTo use the fixtext, press Shift-F<number> or just click the text macro buttons while transceiving.\nHave a lot of fun!\n";

/* --------------------------------------------------------------- */
void create_rxfilehead()
{
    timequery();
    rxfilehead[0] = 0;
    sprintf(rxfilehead, "\n* * * * Start of received data by hfterm * * * *\n"
	"OP: %s \nGMT: %s \n\n", params.pactor.mycall, gmt);
    //fprintf(stderr, "rx file head prepared: %s \n", rxfilehead);
}

void create_rxfilefoot()
{
    timequery();
    rxfilefoot[0] = 0;
    sprintf(rxfilefoot, "\n\n* * * * End of received data by hfterm * * * *\n"
	"OP: %s \nGMT: %s \n", params.pactor.mycall, gmt);
    //fprintf(stderr, "rx file foot prepared: %s \n", rxfilehead);
}

void textinsert(char *filename, GtkText *text)
  {
  	FILE *fi;
  	char buf[16];
  	fi = fopen(filename, "r");
  	if(fi == NULL)
  	  {
  	    fprintf(stderr, "file %s can not be opened.\n", filename);
	    return;
	  }
  	while ( !feof(fi) )
	  {
            memset(buf, 0, sizeof(buf));
	    fgets(buf, 15, fi);
	    gtk_text_insert (text, NULL, NULL, NULL, buf, -1);
	 }
  	fclose(fi);				
 }

/* --------------------------------------------------------------- */

void parse(char* magic, char* bragitem)
{
/* parses fixtexts for macros. Uncomment fprintfs to see ho it 
   works, its greaaat, from scratch by dl4mge -----------------------*/
// if you change this, change char fixmagic, see above
    if(strstr(keytext, magic) != 0)
    {
	char a[2048];
	char *c;
	while(strstr(keytext, magic) != 0)
	{
	  memset(a, 0, sizeof(a));
	  c =  strstr(keytext, magic);
//	  fprintf(stderr, "\n\na anfangs\n%s", a); 
//	  fprintf(stderr, "\ngefunden: %d\n", c - keytext); 
	  strncpy(a, keytext, (c - keytext));
//	  fprintf(stderr, "\n\na nach 1. Teil\n%s", a); 
	  strcat(a, bragitem);
//	  fprintf(stderr, "\n\na nach brag\n%s", a); 
	  strcat(a, c + strlen(magic));
//	  fprintf(stderr, "\n\na nach 2. Teil\n%s", a);
	  strcpy(keytext, a);
//	  printf("\nkeytext:\n%s", keytext);
	}
    }
    return;
}

void fixtext_send(int fixnr)
{
/* Fixtexte senden, ruft Parserfunktion fr call, rig, ant etc. -------*/
	int i = fixnr, l, fixlength;
	char v, *fixtext, fixtextname[8];
	GtkText *txt;
		
	sprintf(fixtextname, "text%hu", i);
	fixlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(Wfixtext), fixtextname)));
	fixtext = gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data(GTK_OBJECT(Wfixtext), fixtextname)), 0, fixlength);
		
	if (fixlength > 0)
	{
	  timequery();
	  strcpy(keytext, fixtext);
	  parse("[CALL]", params.pactor.destcall);
          parse("[PCALL]", params.pactor.destcall);
          parse("[ACALL]", params.amtor.destcall);
          parse("[GCALL]", params.gtor.destcall);
          parse("[MYCALL]", brag.call);
          parse("[OP]", brag.op);
          parse("[QTH]", brag.qth);
          parse("[LOC]", brag.loc);
          parse("[RIG]", brag.rig);
          parse("[PWR]", brag.pwr);
          parse("[ANT]", brag.ant);
          parse("[WWW]", brag.www);
          parse("[TIME]", gmt);
          if (lb.logsize != 0) {
          	parse ("[LTIME]", lb.line[lb.logsize].time);
          	parse ("[LCALL]", lb.line[lb.logsize].call);
          	parse ("[LNAME]", lb.line[lb.logsize].name);
          	parse ("[LQTH]", lb.line[lb.logsize].qth);
          	parse ("[LRSTIN]", lb.line[lb.logsize].rstin);
          	parse ("[LRSTOUT]", lb.line[lb.logsize].rstout);
          	parse ("[LNOTES]", lb.line[lb.logsize].notes);
          	parse ("[LMODE]", lb.line[lb.logsize].mode);
          	parse ("[LBAND]", lb.line[lb.logsize].band);
          	display_status("I replaced magic keys [L...] from log entry Nr. %d", lb.logsize);
          }
          else {
          	display_status("did not parse magic keys from macro concerning log,\n"
          		"because no log entry was specified");
          }
	  txt = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), "textedit"));
	  gtk_text_insert(txt, NULL, NULL, NULL, keytext, strlen(keytext));
	  
	  display_status("transmitting fixtext %hu",  i);
	  for(l = 0; l < strlen(keytext); l++)
	  {
	    	v = keytext[l];
	    	edit_addchar(v);
	  }	
	  edit_newline();
	}
	return;				
}


void fixtext_read()
{
/* - 12 Fix - Texte einlesen ----------------------------------------*/
	FILE *fix;
	int i; 
	char fixfilename[32], fixtextname[8], buf[16];
	for (i = 1; i < 13; i++)
	{
		sprintf(fixfilename, "hf/fix.%02hu", i);
		sprintf(fixtextname, "text%hu", i);
		fix = fopen(fixfilename, "r");
		if(fix == NULL)
		{
		  fprintf(stderr, "fixtextfile %s can not be opened.\n", fixfilename);
		  break;
		}
		
		while ( !feof(fix) )
		{
			memset(buf, 0, sizeof(buf));
			fgets(buf, 15, fix);
			gtk_text_insert ((GTK_TEXT(gtk_object_get_data(GTK_OBJECT(Wfixtext), fixtextname))) , NULL, NULL, NULL, buf, -1);
		}
		fclose(fix);				
	}
}	
  
void fixtext_store()
{
/* - 12 Fix - Texte speichern ---------------------------------------*/
	FILE *fix;
	int i, fixlength;
	char *fixtext, fixfilename[32], fixtextname[8];

	for (i = 1; i < 13; i++)
	{
  	  sprintf(fixfilename, "hf/fix.%02hu", i);
	  sprintf(fixtextname, "text%hu", i);
	  fix = fopen(fixfilename, "w");
	  if(fix == NULL)
	  {
	    fprintf(stderr, "fixtextfile %s can not be opened.\n", fixfilename);
	    return;
	  }
	  fixlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(Wfixtext), fixtextname)));
	  fixtext = gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data(GTK_OBJECT(Wfixtext), fixtextname)), 0, fixlength);
	  if (fixlength > 0)
	  {
	    if ( (fwrite(fixtext, fixlength, 1, fix)) != 1)
	    {
	      fprintf(stderr,	"Error in writing fixtextfile %s\n", fixfilename);
	      return;
	    }  
	    fclose(fix);				
	  }
	}
}


void file_send()
{
/* Datei senden, ohne Parserfunktion -------------------------------------------------*/
	FILE *txfile;
	char c = 0;
	GtkText *txt;

	txt = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), "textedit"));
	fileselection_dialog("choose file for transmit ...");
	txfile = fopen(specialfile, "r");
	if(txfile == NULL) {
	    display_status("transmit file %s can not be opened.", specialfile);
	    return;
	}
	display_status("starting transmission of file %s ...", specialfile);
	edit_newline();
	while (1) {
	    c = fgetc(txfile);      
	    if (!feof(txfile)) {
	    	gtk_text_insert(txt, NULL, NULL, NULL, &c, 1);
    	    edit_addchar(c);
    	    }
    	    else break;
	}	
	edit_newline();
	fclose (txfile);
	display_status("transmission of file %s complete.", specialfile);
	return;				
}


void beacon_send()
{
/* Bake senden ---------------------------------------------------------------------*/
// not yet working
	FILE *beaconfile;
	char c = 0, v = 0, beaconbuffer[512];
	int l, beaconcount = 0, buffercount = 0;
	GtkText *txt;

	memset(beaconbuffer, 0, sizeof(beaconbuffer));
	fileselection_dialog("select small (<512 characters) file for beacon text ...");
	beaconfile = fopen(specialfile, "r");
	if(beaconfile == NULL) {
	    display_status("beacon file %s can not be opened.", specialfile);
	    return;
	}

	display_status("will read (part of) beaconfile %s into buffer", specialfile);
	display_status("empty buffer contains: \n%s", beaconbuffer);
	
	while (buffercount < sizeof(beaconbuffer) - 1) {
	    c = fgetc(beaconfile);
    	    if (!feof(beaconfile)) {
		beaconbuffer[buffercount] = c;
		buffercount++;
	    }
	    else break;
    	}
    	

//	while ( !feof(beaconfile) ){
//	fgets(beaconbuffer, sizeof(beaconbuffer), beaconfile);
//	}
	fclose(beaconfile);				

	display_status("have read (part of) beaconfile %s into buffer", specialfile);
	display_status("buffer contains: \n%s", beaconbuffer);
	display_status("! Will start beacon transmission from file %s ... !\n"
		"! Please do not disturb your ham friends !"
		"! Please do not let your rig unattended !\n"
		"! You can stop the beacon with <Alt>b !", specialfile);
		
	txt = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), "textedit"));
	while (beacon_on == 1) {
	    gtk_text_insert (txt, NULL, NULL, NULL, beaconbuffer, strlen(beaconbuffer));
	    // oder -1 ?
	    display_status(".");
	    beaconcount++;

       	    edit_newline();
	    for(l = 0; l < strlen(beaconbuffer); l++) {
	    	v = beaconbuffer[l];
		edit_addchar(v);
	    }	
	    edit_newline();
	    display_status("Beacon count: %d ...", beaconcount);
	    if (beaconcount / 10 == 0) {
		display_status("! Beacon transmission from file %s ... !\n"
			"! Please do not disturb your ham friends !"
			"! Please do not let your rig unattended !\n"
			"! You can stop the beacon with <Alt>b !", specialfile);
	    }
	    // duration of tx at 45 bd plus one sec
// usleep freezes
//	    usleep ( ((60000000 / 45) * strlen(beaconbuffer))  + 1000000);
// also system sleep freezes
//	    system (" sleep 5");
// this makes no sleep , hang
//	    	    nanosleep ((((60000000000 / 45) * strlen(beaconbuffer))  + 1000000000), NULL);
	}
	display_status("Beacon transmission stopped.");
	display_status("Transmitted %d copies of file %s complete.", 
		beaconcount, specialfile);
	return;				
}


void rx_routine_store_prepare()
{
/* - Rx - Text vorbereiten   --------------------------------------------*/
	//wenn viiiiiiiele rx-files gewnscht:
	//sprintf(rxfilename, "hf/hfrx.%hu", getpid());
	//oder statt getpid timestamp / gmt
	//wenn 5 alte hfrx-files gelagert werden sollen:
	FILE *rxfile;
	char *rxfilename =  "hf/hfrx";

	remove("hf/hfrx.old.01");
  	rename("hf/hfrx.old.02", "hf/hfrx.old.01");	  
  	rename("hf/hfrx.old.03", "hf/hfrx.old.02");
	rename("hf/hfrx.old.04", "hf/hfrx.old.03");	  
  	rename("hf/hfrx.old.05", "hf/hfrx.old.04");
	rename("hf/hfrx", "hf/hfrx.old.05");	  

	create_rxfilehead();
	rxfile = fopen(rxfilename, "w");
	if(rxfile == NULL) {
	    fprintf(stderr, "rx-file %s can not be opened first time.\n", rxfilename);
	    return;
	}
	if ( (fwrite(rxfilehead, strlen(rxfilehead), 1, rxfile)) != 1) {
    	    fprintf(stderr,
		"Error in writing head lines to new routine rx-file %s\n", 
		rxfilename);
        return;
	}
	fclose(rxfile);
}

void rx_routine_store_part()
{
/* - Rx - Text bei laufendem Programm teilweise (hier: halb) speichern ------*/
	FILE *rxfile;
	char *rxfilename =  "hf/hfrx";
	char *rxtxt;
	int rxlength = 0;
	rxfile = fopen(rxfilename, "a");
	if(rxfile == NULL)
	  {
	    fprintf(stderr,	"rx-file %s can not be opened.\n", rxfilename);
	    return;
	  }
	rxlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), 
	    "textmain")));
	rxtxt = gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data(GTK_OBJECT(wmain), 
	    "textmain")), 0, rxlength);
	if ( (fwrite(rxtxt, MAXRX / 2, 1, rxfile)) != 1)
	  {
	    fprintf(stderr,	"Error in writing rx-file %s\n", rxfilename);
	    return;
	  }
//	fwrite("\n", strlen("\n"), 1, rxfile);
	fclose(rxfile);	
//	display_status("Older half of Rx Data stored in ~/hf/hfrx\n");
	return;
}

void rx_routine_store_rest()
{
/* - Rx - Text vor Beenden speichern ---------------------------------------*/
	FILE *rxfile;
	char *rxfilename =  "hf/hfrx";
	char *rxtxt;
	GtkText *txt;
	int rxlength;

	rxlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), 
	    "textmain")));
	if (rxlength > 0)
	{
	  rxfile = fopen(rxfilename, "a");
	  if(rxfile == NULL)
	  {
	    fprintf(stderr,	"rx-file %s can not be opened.\n", rxfilename);
	    return;
	  }
	 
	txt = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain),"textmain"));
	gtk_text_freeze(txt);
	
	rxtxt = gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data(GTK_OBJECT(wmain), 
	      "textmain")), 0, rxlength);
	  
	if ( (fwrite(rxtxt, rxlength, 1, rxfile)) != 1) {
	    fprintf(stderr,	"Error in writing rx-file %s\n", rxfilename);
	    return;
	}
	create_rxfilefoot();	
	fwrite(rxfilefoot, strlen(rxfilefoot), 1, rxfile);
	fclose(rxfile);	
	fprintf(stderr, "Rest of Rx Data stored in %s\n", rxfilename);
	return;
	}
	else
	{
	  fprintf(stderr, "You had no rx - you get no rx-file.\n");
	  return;
	}  
}

/* - fileselection universal---------------------------------------------- */

void fileselection_dialog(char *title)
{
	GtkWidget *filedialog;
//there should be defined global in this file a char specialfile[256]; 
//long array because maybe with path.
	memset(specialfile, 0, sizeof(specialfile));	
//	display_status("specialfilereset  o.k.: %s\n", specialfile);
	filedialog = gtk_file_selection_new(title);
	gtk_window_set_modal(GTK_WINDOW(filedialog), TRUE);
		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
	    "clicked", (GtkSignalFunc) fileselection, filedialog);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
	    "clicked", (GtkSignalFunc) fileselection_cancel, filedialog);
	gtk_signal_connect(GTK_OBJECT(filedialog),
	    "destroy", (GtkSignalFunc) fileselection_cancel, filedialog);
//	fprintf(stderr, "widget filedialog created.\n");
//	display_status("widget filedialog created.\n");
	gtk_widget_show(filedialog);
//	display_status("after showing file selection menu \n");
//	fprintf(stderr, "after showing file selection menu \n");
	gtk_main();
//	display_status("after gtk-main \n");
//	fprintf(stderr, "after gtk-main \n");
//	fprintf(stderr, "specialfile still o.k.: %s\n", specialfile);
//	display_status("specialfile still o.k.: %s\n", specialfile);
}

void fileselection(GtkWidget *widget, GtkWidget * filedialog)
{
	char *selectedname = NULL;
		
	selectedname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filedialog));
//	fprintf(stderr, "fileselection o.k.: %s\n", selectedname);
	if ((strlen(selectedname) > 0) && (strlen(selectedname) < 64))  {
	    display_status("filename length between 1 and 64, o.k.: %s", selectedname);
	    sleep (1);
	    if (selectedname[strlen(selectedname) -1] == 47) {
	 	//sleep(1);   
		display_status("You selected %s, it's a directory! Try again!", 
		    selectedname);
	    }
	    else {
	        //display_status("You selected %s, o.k.", selectedname);
	        strcpy(specialfile, selectedname); 
	        display_status("You selected %s, o.k.", specialfile);
	    }
	}
	gtk_widget_destroy(filedialog);
   	gtk_main_quit();
}

void fileselection_cancel(GtkWidget *widget, GtkWidget * filedialog)
{
    gtk_widget_destroy(filedialog);
    gtk_main_quit();
}

/* - fileselection ------------------------------------------------------- */

void rx_demand_store()
{
/* - Dateiauswahlmen zum Speichern des Rx-Testes -------------------------*/
    GtkText *txt;
    FILE *rxfile;
    char *rxtxt = NULL;
    int rxlength = 0;
    
    txt = GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain),"textmain"));
    rxlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), 
	"textmain")));
    if (rxlength == 0) {
    	// this no no stuff for test of display_status.....
	display_status("You had %s rx - you can get %s rx-file.", "no", "no");
    	return;
    }
    else {
    	display_status("You received something ... choose a file...");
	fileselection_dialog("Save rx input as file ...");
    }
    if (specialfile[0] == 0) {
    	fprintf(stderr,	"rxfilename %s not yet specified.\n", specialfile);
        return;
    }
    fprintf(stderr, "rx-file %s has been prepared.\n", specialfile);
    display_status("rx-file %s has been prepared.", specialfile);
    rxfile = fopen(specialfile, "w");
    if (rxfile == NULL){
        fprintf(stderr,	"rx-file %s can not be opened.\n", specialfile);
        return;
    }
    gtk_text_freeze(txt);
    rxlength = gtk_text_get_length(GTK_TEXT(gtk_object_get_data(GTK_OBJECT(wmain), 
	"textmain")));
    rxtxt = gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data(GTK_OBJECT(wmain), 
	"textmain")), 0, rxlength);
    create_rxfilehead();
    if ( (fwrite(rxfilehead, strlen(rxfilehead), 1, rxfile)) != 1) {
        fprintf(stderr,	
	    "Error in writing head lines to choosen rx-file %s\n", 
	    specialfile);
        return;
    }
    if ( (fwrite(rxtxt, rxlength, 1, rxfile)) != 1) {
        fprintf(stderr,	
	    "Error in writing rx-text to choosen rx-file %s\n", 
	    specialfile);
        return;
    }
    create_rxfilefoot();
    if ( fwrite(rxfilefoot, strlen(rxfilefoot), 1, rxfile)!= 1) {
        fprintf(stderr,	
	    "Error in writing rx-text to choosen rx-file %s\n", 
	    specialfile);
        return;
    }
    fclose(rxfile);	
    gtk_text_thaw(txt);
    display_status("rx-file %s has been written.", specialfile);
    fprintf(stderr, "Rx Data stored in %s\n", specialfile);    
    return;
}	

